import { Context } from 'skypager-registry'
import Helper from '../helper'

export class DocumentType extends Helper {
  static isCacheable = true

  static cacheable(setting = true) {
    return this.isCacheable = !!setting
  }

  static createRegistry = (options = {}) => new Context('documentTypes', {
    context: require.context('../document-types', false, /.js$/),
    ...options,
    sourceModule: {
      id: (module && module.id) || __filename,
      filename: __filename, dirname: __dirname,
      ...options.sourceModule || {},
    },
  })

  static attach(project, options = {}) {
    return Helper.attach(project, DocumentType, {
      registryProp: 'documentTypes',
      lookupProp: 'documentType',
      registry: options.registry || DocumentType.createRegistry(),
      ...options,
    })
  }

  get matchingDocuments() {
    return this.loadMatchingDocuments({ready: false})
  }

  loadMatchingDocuments(options = {}) {
    const { 
      ready = true,
      pattern = '*'
    } = options

    return this.project
      .route(pattern)
      .filter(r => this.testDocument(r.doc))
      .map(({doc}) =>  ready ? doc.ready : doc)
  }

  testDocument(doc) {
    return this.provider.testDocument(doc)
  }

  decorate(doc) {
    return this.provider.decorate(doc)
  }
  
   markdownTransforms() {
      return {
        markdown: require('../transformers/markdown').default,
        retext: require('../transformers/retext').default,
        rehype: require('../transformers/rehype').default,
      }
    }

  markdownUtils() {
    return {
      filter: require('unist-util-filter'),
      findAfter: require('unist-util-find-after'),
      findAllAfter: require('unist-util-find-all-after'),
      findAllBefore: require('unist-util-find-all-before'),
      index: require('unist-util-index'),
      map: require('unist-util-map'),
      parents: require('unist-util-parents'),
      position: require('unist-util-position'),
      removePosition: require('unist-util-remove-position'),
      select: require('unist-util-select'),
      source: require('unist-util-source'),
      visit: require('unist-util-visit'),
      reporter: require('vfile-reporter'),
      rehypeDocument: require('rehype-document'),
      toString: require('mdast-util-to-string'),
      docblockParser: require('docblock-parser')
    }
  }
}

export default DocumentType
