import { parser } from './markdown'
import remarkToRehype from 'remark-rehype'
import stringify from 'rehype-stringify'

export default (options = {}, context = {}) => {
  const { profile = 'default' } = options
  return (profiles[profile] || nestedSections)(options)
}

export const applyTo = (input, options) => {
  const { method = 'parse', parser = 'nested' } = options
  return profiles[parser](options)[method](input)
}

export const rehype = (options = {}) =>
  parser(options)
    .use(remarkToRehype)
    .use(stringify)

export const nestedSections = (options = {}) =>
  rehype(options)
    .use(() => nestSections)
    .use(stringify)

export const profiles = {
  default: nestedSections,
  nested: nestedSections,
  standard: rehype
}

/* eslint-disable */

// Thanks to titus @wooorm the author of remark / retext / rehype for this!
const nestSections = (tree) => {
  const result = [];
  let section;
  let article;

  tree.children.forEach(function (child) {
    var ctx;

    if (child.tagName === 'h2') {
      ctx = result;
      article = null;
      section = child = h('section', [child]);
    } else if (child.tagName === 'h3') {
      ctx = section ? section.children : result;
      article = child = h('article', [child]);
    } else {
      ctx = (article || section || {}).children || result;
    }

    ctx.push(child);
  });

  tree.children = [h('main', result)];
}
