require('./shell')

const express = require('express')
const webpack = require('webpack')
const devMiddleware = require('webpack-dev-middleware')
const hotMiddleware = require('webpack-hot-middleware')
const DllBundle = require('../devpacks/web.benzinga-react.dll')

const PORT = argv.port || process.env.PORT || 8080
const HOSTNAME = argv.host || process.env.HOST || '0.0.0.0'

banner()

mkdir('-p', 'tmp/cache/webpack')

log(`${emoji('dancer')}  Building development configuration`)
_(DllBundle).mapValues(({name,entry}) => log(`      DLL: ${name} packages ${entry.length} dependencies`))

const config = require('../webpacks/base.babel')('development')

log(`${emoji('package')}  Creating Webpack Compiler`)

_(DllBundle).mapValues(({name,entry}) => log(`      DLL: ${name} packages ${entry.length} dependencies`))

const compiler = webpack({
  ...config,
})

const app = express()

const middleware = devMiddleware(compiler, {
  publicPath: config.output.publicPath || '/',
  noInfo: false,
  quiet: false,
  stats: "normal",
})

// middleware uses memory-fs which is faster. we use this to serve index.html
const fs = middleware.fileSystem;

app.use(express.static(compiler.outputPath))
app.use(express.static(join(process.cwd(),'tmp','devpacks')))

app.use(middleware)
app.use(hotMiddleware(compiler))

app.get('*', (req, res) => {
  fs.readFile(join(compiler.outputPath, 'index.html'), (err, file) => {
    if (err) {
      res.sendStatus(404);
    } else {
      res.send(file.toString());
    }
  });
});

compiler.plugin('compilation', function(compilation) {
  global.compilation = compilation

  if (argv.repl && !global.replServer) {
    global.replServer = require('./repl')(
      { icon: 'sunny' },
      { app,
        getCompilation: (key) => get(global, `compilation.${key}`),
        getCompiler: (key) => get(compiler,key),
        lodash: require('lodash'),
      }
    )

    setTimeout(() => replServer.displayPrompt(), 400)
  }
})

log(`${emoji('rocket')}  Launching that Hot Reloading Dev Server...`)

app.listen(PORT, HOSTNAME, (err) => {
  if (err) {
    log(`${emoji('umbrella')}  Error starting server:`)
    log(err.message)
    process.exit(1)
  } else {
    log()
    log(`${emoji('rocket')}  Starting server on ${HOSTNAME}:${PORT}`)

    if(argv.repl) {
      log(`${emoji('coffee')}  The REPL will launch when Webpack finishes compiling.`)
    }
  }
})
