import React from 'react';
import classnames from 'classnames';
import Immutable from 'immutable';
import TutorialStep from './TutorialStep';
import TutorialOverlay from './TutorialOverlay';
import ProgressBar from 'components/ui/ProgressBar';
import { completeTutorial, saveTutorials } from 'actions/tutorials';

export default class NewsfeedTutorial extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      step: 1,
      title: null,
      body: null,
      shape: null,
      x: 0,
      y: 0,
      s: 1
    };
  }
  componentDidMount() {
    this.goToStep(this.state.step);
  }
  componentDidUpdate(prevProps, prevState) {
    if (prevState.step !== this.state.step) this.goToStep(this.state.step);
  }
  nextStep() {
    this.setState({
      step: this.state.step + 1
    });
  }
  goToStep(step) {
    let x = null;
    let y = null;
    let s = null;
    let stepPosHorizontal = null;
    let stepPosVertical = null;
    let title = null;
    let body = null;
    let hidden = null;

    switch (step) {
    case 1:
      x = 0;
      y = 0;
      s = 0;
      hidden = true;
      stepPosHorizontal = 'left';
      stepPosVertical = 'bottom';
      title = (<h4>Welcome to your first Newsfeed!</h4>);
      body = (<p>
                The Newsfeed widget streams economic news from Benzinga & other media sources
                and from press releases.
                <br />
                <br />
                In the search bar you can search by one keyword and/or multiple ticker symbols, and
                filter your results further by channel, source, and watchlist.
                <br />
                <br />
                The widget ( <strong><i className="bzpro-icon-options"></i></strong> ) settings
                holds customization options for this Newsfeed. You can edit the name of the widget,
                and adjust the text-size of and spacing between headlines. Apply highlight themes to different
                channels and sources to quickly distinguish what topics headlines cover and from where they originate.
              </p>);
      break;
    default:
      completeTutorial('newsfeed', this.props.tutorials);
      break;
    }
    this.setState({
      title,
      body,
      x,
      y,
      s,
      hidden
    });
  }
  render() {
    const { step, title, body, x, y, s, hidden } = this.state;
    const numSteps = 1;
    return (
      <div className="tut">
        <TutorialStep tutorial="newsfeed" title={title} body={body} currentStep={step} numSteps={numSteps} nextStep={() => this.nextStep()} />
        <TutorialOverlay x={x} y={y} s={s} hidden={hidden}/>
      </div>
    );
  }
}

NewsfeedTutorial.propTypes = {
  tutorials: React.PropTypes.object
};
